package com.example.fio.td4;

import android.content.Intent;
import android.os.AsyncTask;
import android.os.Bundle;
import android.support.design.widget.FloatingActionButton;
import android.support.design.widget.Snackbar;
import android.support.v7.app.AppCompatActivity;
import android.support.v7.widget.Toolbar;
import android.view.View;
import android.widget.EditText;
import android.widget.ScrollView;
import android.widget.TextView;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.PrintWriter;
import java.net.Socket;


public class ChatActivity extends AppCompatActivity {

    private final static String STATE_CHATLOG = "STATE_CHATLOG";
    private ReadMessages readMessages;



    private TextView chat;
    private EditText entry;
    private ScrollView scroll;

    private String chatLog = "";
    private String login;

    private PrintWriter writer = new PrintWriter(System.out, true);
    private BufferedReader reader = new BufferedReader(new InputStreamReader(System.in));

    private class StartNetwork extends AsyncTask<Void, Void, Boolean> {

        @Override
        protected Boolean doInBackground(Void... v) {
            try {
                Socket socket = new Socket("172.18.73.43", 7777);
                writer = new PrintWriter(socket.getOutputStream(), true);
                writer.println("LOGIN " + login);
                reader = new BufferedReader(new InputStreamReader(socket.getInputStream()));
                return true;
            } catch (IOException e) {
                return false;
            }
        }

        @Override
        protected void onPostExecute(Boolean b) {
            if (b) {
                displayMessage("Connected to server\n");
                readMessages = new ReadMessages();
                readMessages.execute();
            } else
                displayMessage("Could not connect to server\n");
        }
    }

    private class ReadMessages extends AsyncTask<Void, String, Void> {
        @Override
        protected Void doInBackground(Void... v) {
            while (!isCancelled())
                try {
                    String message = reader.readLine();
                    publishProgress(message);
                } catch (IOException e) {
                    break;
                }
            return null;
        }

        @Override
        protected void onProgressUpdate(String... messages) {
            displayMessage(messages[0] + "\n");
        }
    }


    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_chat);
        Toolbar toolbar = (Toolbar) findViewById(R.id.toolbar);
        setSupportActionBar(toolbar);

        chat = (TextView) findViewById(R.id.chat);
        entry = (EditText) findViewById(R.id.entry);
        scroll = (ScrollView) findViewById(R.id.scroll);
        Intent data = getIntent();
        login = data.getStringExtra(TD4Activity.EXTRA_LOGIN);

        if (savedInstanceState != null) {
            String log = savedInstanceState.getString(STATE_CHATLOG);
            displayMessage(log);
        }


        writer.println("onCreate ChatActivity");
    }


    @Override
    protected void onStart() {
        super.onStart();
        new StartNetwork().execute();
    }


    private void displayMessage(String message) {
        // On ajoute ce message a l'ensemble du chat
        chatLog += message;
        // On affiche le chat
        chat.setText(chatLog);
        // On scrolle en bas pour visualiser le dernier message
        scroll.fullScroll(View.FOCUS_DOWN);
    }


    public void enter(View v) {
        // On récupère le texte entré par l'utilisateur
        String text = entry.getText().toString();
        // S'il n'est pas vide
        if (!text.isEmpty()) {
            writer.println("SEND " + text);
//            //On ajoute le login de l'utilisateur avant et un saut de ligne après
//            String message = login + ": " + text + "\n";
//            // On affiche le message
//            displayMessage(message);
            // On nettoie la zone de texte d'entrée
            entry.setText("");
        }
    }


    public void logout(View v) {
        writer.println("LOGOUT");
        readMessages.cancel(true);
        finish();
    }

    @Override
    public void onSaveInstanceState(Bundle savedInstanceState) {
        savedInstanceState.putString(STATE_CHATLOG, chatLog);
        super.onSaveInstanceState(savedInstanceState);
    }





}



